#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

// Define the OLED display width and height
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64

// Define I2C pins for the SAMD21
#define SDA_PIN 16
#define SCL_PIN 17

// Create an instance of the SSD1306 display
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

// Define pins for the rotary encoder and switch
#define CLK_PIN 18  // B pin of the encoder
#define DT_PIN 22   // A pin of the encoder
#define SW_PIN 19   // Pin connected to the switch

int counter = 0;  // Counter to keep track of encoder position
int currentStateCLK;
int lastStateCLK;
int currentStateSW;
int lastStateSW = HIGH; // Assume switch is not pressed initially
unsigned long lastDebounceTime = 0;  // The last time the switch input pin was toggled
unsigned long debounceDelay = 50;    // The debounce time in milliseconds

void setup() {
  // Set encoder pins as inputs
  pinMode(CLK_PIN, INPUT);
  pinMode(DT_PIN, INPUT);
  pinMode(SW_PIN, INPUT_PULLUP);  // Enable internal pull-up resistor for the switch

  // Initialize serial communication
  Serial.begin(9600);

  // Set Wire library's default SDA and SCL pins
  Wire.begin(SDA_PIN, SCL_PIN);

  // Initialize the display with the I2C address
  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {  // 0x3C is the I2C address for most 128x64 OLED displays
    Serial.println(F("SSD1306 allocation failed"));
    for (;;);  // Don't proceed, loop forever
  }

  // Clear the buffer
  display.clearDisplay();

  // Read the initial state of CLK
  lastStateCLK = digitalRead(CLK_PIN);

  // Display initial counter value
  displayCounter();
}

void loop() {
  // Read the current state of CLK
  currentStateCLK = digitalRead(CLK_PIN);

  // If the previous and current state of CLK are different, then a pulse occurred
  if (currentStateCLK != lastStateCLK && currentStateCLK == HIGH) {
    // Read the DT state
    int currentStateDT = digitalRead(DT_PIN);

    // If the DT state is different to the CLK state then the encoder is rotating counterclockwise
    if (currentStateDT != currentStateCLK) {
      counter--;
    } else {
      // Encoder is rotating clockwise
      counter++;
    }

    // Print the current counter value to the serial monitor
    Serial.print("Counter: ");
    Serial.println(counter);

    // Display the updated counter value on the OLED
    displayCounter();
  }

  // Update lastStateCLK with the current state
  lastStateCLK = currentStateCLK;

  // Read the current state of the button
  int reading = digitalRead(SW_PIN);

  // Check if the button state has changed (with debounce)
  if (reading != lastStateSW) {
    lastDebounceTime = millis();
  }

  if ((millis() - lastDebounceTime) > debounceDelay) {
    // If the button state has been stable for debounceDelay milliseconds
    if (reading != currentStateSW) {
      currentStateSW = reading;

      if (currentStateSW == LOW) {
        // Button is pressed
        Serial.println("Button pressed");
        // Update OLED display when button is pressed
        displayButtonState("Pressed");
      } else {
        // Button is released
        Serial.println("Button released");
        // Update OLED display when button is released
        displayButtonState("Released");
      }
    }
  }

  // Update the last button state
  lastStateSW = reading;
}

// Function to display the counter value on the OLED
void displayCounter() {
  // Clear the buffer
  display.clearDisplay();

  // Set text size and color
  display.setTextSize(2);
  display.setTextColor(SSD1306_WHITE);

  // Set cursor position
  display.setCursor(0, 0);

  // Display the counter value
  display.print("Counter: ");
  display.println(counter);

  // Update the display with the buffer contents
  display.display();
}

// Function to display button state on the OLED
void displayButtonState(String state) {
  // Clear the buffer
  display.clearDisplay();

  // Set text size and color
  display.setTextSize(1);
  display.setTextColor(SSD1306_WHITE);

  // Set cursor position
  display.setCursor(0, 0);

  // Display the button state
  display.print("Button: ");
  display.println(state);

  // Update the display with the buffer contents
  display.display();
}
